/****  Virtual Oil Drop Panel

  Use analogRead to measure potentiometer as a voltage divider
  to simulate voltage in Virtual Millikan Oil Drop
  Use 3 buttons to simulate controls: power, polarity, reset/exit

  revision history:
  09/22/2015 Mark D. Shattuck <mds> VMon.ino
             Lab 2 part II for P371
             000 mds Starting point for assignment 2-II
  12/22/2020 000 mds convert to SimpleGate
  03/08/2020 000 mds convert to  VirtualOil Drop Panel

  Therory of opperation:
  Use potentiometer as 5V voltage divider

           potentiometer
  5V-------|||||||||||||-------GND
             ^
             |
            A1

  A1 on sweep

**************/

// function to wait for a specified character from the serial port.
bool waitFor(char in, unsigned int pollTime = 256, unsigned long timeout = 1e6) {
  bool done = false;
  char c;
  unsigned long T;
  T = micros();
  while (!done && (micros() - T < timeout)) {
    delayMicroseconds(pollTime);
    if (Serial.available() > 0) {
      c = Serial.read();
      done = (c == in);
    }
  }
  return done;
}

int pinIn = A1;  // potentiometer pin
int pinPow = 2;  // power ON/OFF
int pinPol = 3;  // polarity +/-
int pinRes = 4;  // RESET charge (hold >3sec for EXIT)

int voltage = 0; // variable to store the value coming from pinIn

/* setup bitmap variable to store button state info
    bstate 0bXXXXWPRE
    W -> power state ON/OFF
    P -> polarity state 1=>(+) / 0=>(-1)
    R -> RESET state RESET/!RESET
    E -> EXIT state EXIT/!EXIT
    X -> future use
*/

#define BPOW 0b00001000
#define BPOL 0b00000100
#define BRES 0b00000010
#define BXIT 0b00000001

byte bstate = BPOL;


// variables for power button
int pw;            // state of power button
int ppw = 1;       // previous state of power button

// variables for polarity button
int pol;           // state of polarity button
int ppol;          // previous state of polarity button

// variables for RESET/EXIT button
int res;           // state of reset button
int pres = 1;      // previous state of reset button

int cres;          // measure of time reset held down
int cexit = 15000; // ~ 3 sec held down

unsigned long T;     // Time for delay print
long del = 1000000;  // Delay between prints in us

void setup() {
  Serial.begin(115200);              // Set up Serial library at 115200 bps

  // INPUT_PULLUP allows buttons without resistors
  pinMode(pinPow, INPUT_PULLUP);     // Set up power pin
  pinMode(pinPol, INPUT_PULLUP);     // Set up polarity pin
  pinMode(pinRes, INPUT_PULLUP);     // Set up reset/exit pin

  pinMode(13, OUTPUT);      // initialize pin PIN as an output.
  while (!waitFor('?'));    // stop until recieve '?' from serial port
  Serial.println('K');      // respond O('K')
  digitalWrite(13, HIGH);   // visual response

}

void loop() {
  // pinIn count will represent volage across cell
  voltage = analogRead(pinIn);       // read 10-bit A/D from A1 0 - 2^10-1=1023

  // process power pin ON/OFF toggle on press
  pw = digitalRead(pinPow);
  if (pw == 0 && ppw == 1) bstate ^= BPOW;
  ppw = pw;

  // process polarity pin +/- toggle on press
  pol = digitalRead(pinPol);
  if (pol == 0 && ppol == 1) bstate ^= BPOL;
  ppol = pol;

  // process RESET pin one-off on release
  res = digitalRead(pinRes);
  if (pres == 0 && res == 1 && (bstate&BXIT)==0) bstate ^= BRES; 
  pres = res;

  // process EXIT pin hold ~3sec
  cres += 1 - 4 * digitalRead(pinRes);  // (+1 while press) (-3 while release)
  if (cres < 0) cres = 0;
  if (cres >= cexit) {
    bstate |= BXIT;
    cres = cexit;
  }

  while (Serial.available() > 0) {
    if (Serial.read() == 'g') {
      Serial.print(voltage, HEX);
      Serial.println(bstate, HEX);
      if (bstate & BRES) bstate &= ~BRES;
    }
  }
}
